@@ -1,4 +1,5 @@
 import json
+import logging
 import os
 import sys
 import time
@@ -10,9 +11,11 @@ from .state import STATE
 from .find_julia import julia_version, find_julia
 from .install_julia import log
 
+logger = logging.getLogger('juliapkg')
+
 ### META
 
-META_VERSION = 1 # increment whenever the format changes
+META_VERSION = 2  # increment whenever the format changes
 
 def load_meta():
     fn = STATE['meta']
@@ -26,6 +29,7 @@ def save_meta(meta):
     assert isinstance(meta, dict)
     assert meta.get('meta_version') == META_VERSION
     fn = STATE['meta']
+    os.makedirs(os.path.dirname(fn), exist_ok=True)
     with open(fn, 'w') as fp:
         json.dump(meta, fp)
 
@@ -82,42 +86,52 @@ def can_skip_resolve():
     # resolve if we haven't resolved before
     deps = load_meta()
     if deps is None:
+        logger.debug('no meta file')
         return False
     # resolve whenever Julia changes
-    exe = deps.get("executable")
-    if exe is None:
-        return False
-    ver = deps.get("version")
+    exe = deps["executable"]
+    ver = deps["version"]
     exever = julia_version(exe)
-    if ver is None or exever is None or ver != str(exever):
+    if exever is None or ver != str(exever):
+        logger.debug('changed version %s to %s', ver, exever)
+        return False
+    # resolve when going from offline to online
+    offline = deps['offline']
+    if offline and not STATE['offline']:
+        logger.debug('was offline now online')
         return False
     # resolve whenever swapping between dev/not dev
-    isdev = deps.get("dev")
-    if isdev is None or isdev != STATE["dev"]:
+    isdev = deps['dev']
+    if isdev != STATE["dev"]:
+        logger.debug('changed dev %s to %s', isdev, STATE['dev'])
         return False
     # resolve whenever anything in sys.path changes
-    timestamp = deps.get("timestamp")
-    if timestamp is None:
-        return False
+    timestamp = deps["timestamp"]
     timestamp = max(os.path.getmtime(STATE["meta"]), timestamp)
-    sys_path = deps.get("sys_path")
-    if sys_path is None or sys_path != sys.path:
+    sys_path = deps["sys_path"]
+    if sys_path != sys.path:
+        logger.debug('sys.path changed %s to %s', sys_path, sys.path)
         return False
     for path in sys.path:
-        if not path:
+        here = not path
+        if here:
             path = os.getcwd()
         if not os.path.exists(path):
             continue
-        if os.path.getmtime(path) > timestamp:
+        if (not here) and (os.path.getmtime(path) > timestamp):
+            logger.debug('directory changed %r', path)
             return False
         if os.path.isdir(path):
             fn = os.path.join(path, "juliapkg.json")
             if os.path.exists(fn) and os.path.getmtime(fn) > timestamp:
+                logger.debug('file changed %r', fn)
                 return False
     return deps
 
 def deps_files():
     ans = []
+    # the default deps file
+    ans.append(cur_deps_file())
     # look in sys.path
     for path in sys.path:
         if not path:
@@ -129,13 +143,6 @@ def deps_files():
         for subdir in os.listdir(path):
             fn = os.path.join(path, subdir, "juliapkg.json")
             ans.append(fn)
-    # {prefix}/juliapkg.json
-    if STATE['prefix']:
-        fn = os.path.join(STATE['prefix'], 'juliapkg.json')
-        ans.append(fn)
-    # ~/.pyjuliapkg.json
-    fn = os.path.join(os.path.expanduser('~'), '.pyjuliapkg.json')
-    ans.append(fn)
     return list(set(os.path.normcase(os.path.normpath(os.path.abspath(fn))) for fn in ans if os.path.isfile(fn)))
 
 def required_packages():
@@ -247,19 +254,20 @@ def resolve(force=False, dry_run=False):
             if pkg.version:
                 print(f'{pkg.name} = "{pkg.version}"', file=fp)
         print(file=fp)
-    # install the packages
-    dev_pkgs = ', '.join([pkg.jlstr() for pkg in pkgs if pkg.dev])
-    add_pkgs = ', '.join([pkg.jlstr() for pkg in pkgs if not pkg.dev])
-    script = ['import Pkg']
-    if dev_pkgs:
-        script.append(f'Pkg.develop([{dev_pkgs}])')
-    if add_pkgs:
-        script.append(f'Pkg.add([{add_pkgs}])')
-    script.append('Pkg.resolve()')
-    log(f'Installing packages:')
-    for line in script:
-        log('julia>', line, cont=True)
-    run([exe, '--project='+project, '-e', '; '.join(script)], check=True)
+    if not STATE['offline']:
+        # install the packages
+        dev_pkgs = ', '.join([pkg.jlstr() for pkg in pkgs if pkg.dev])
+        add_pkgs = ', '.join([pkg.jlstr() for pkg in pkgs if not pkg.dev])
+        script = ['import Pkg']
+        if dev_pkgs:
+            script.append(f'Pkg.develop([{dev_pkgs}])')
+        if add_pkgs:
+            script.append(f'Pkg.add([{add_pkgs}])')
+        script.append('Pkg.resolve()')
+        log(f'Installing packages:')
+        for line in script:
+            log('julia>', line, cont=True)
+        run([exe, '--project='+project, '--startup-file=no', '-e', '; '.join(script)], check=True)
     # record that we resolved
     save_meta({
         "meta_version": META_VERSION,
@@ -269,6 +277,7 @@ def resolve(force=False, dry_run=False):
         "timestamp": time.time(),
         "sys_path": sys.path,
         "pkgs": [pkg.dict() for pkg in pkgs],
+        "offline": bool(STATE['offline']),
     })
     STATE['resolved'] = True
     STATE['executable'] = exe
@@ -283,14 +292,18 @@ def project():
     resolve()
     return STATE['project']
 
-def cur_deps_file():
-    if STATE['prefix']:
-        return os.path.join(STATE['prefix'], 'juliapkg.json')
+def cur_deps_file(target=None):
+    if target is None:
+        return STATE['deps']
+    elif os.path.isdir(target):
+        return os.path.abspath(os.path.join(target, 'juliapkg.json'))
+    elif os.path.isfile(target) or (os.path.isdir(os.path.dirname(target)) and not os.path.exists(target)):
+        return os.path.abspath(target)
     else:
-        return os.path.join(os.path.expanduser('~'), '.pyjuliapkg.json')
+        raise ValueError('target must be an existing directory, or a file name in an existing directory')
 
-def load_cur_deps():
-    fn = cur_deps_file()
+def load_cur_deps(target=None):
+    fn = cur_deps_file(target=target)
     if os.path.exists(fn):
         with open(fn) as fp:
             deps = json.load(fp)
@@ -298,19 +311,20 @@ def load_cur_deps():
         deps = {}
     return deps
 
-def write_cur_deps(deps):
-    fn = cur_deps_file()
+def write_cur_deps(deps, target=None):
+    fn = cur_deps_file(target=target)
     if deps:
+        os.makedirs(os.path.dirname(fn), exist_ok=True)
         with open(fn, 'w') as fp:
             json.dump(deps, fp)
     else:
         if os.path.exists(fn):
             os.remove(fn)
 
-def status():
+def status(target=None):
     res = resolve(dry_run=True)
     print('JuliaPkg Status')
-    fn = cur_deps_file()
+    fn = cur_deps_file(target=target)
     if os.path.exists(fn):
         with open(fn) as fp:
             deps = json.load(fp)
@@ -339,8 +353,8 @@ def status():
         for (name, info) in pkgs.items():
             print(f'  {name}: {info}')
 
-def set_julia_compat(compat):
-    deps = load_cur_deps()
+def require_julia(compat, target=None):
+    deps = load_cur_deps(target=target)
     if compat is None:
         if 'julia' in deps:
             del deps['julia']
@@ -350,13 +364,13 @@ def set_julia_compat(compat):
         elif not isinstance(compat, Compat):
             raise TypeError
         deps['julia'] = str(compat)
-    write_cur_deps(deps)
+    write_cur_deps(deps, target=target)
     STATE['resolved'] = False
 
-def add(pkg, *args, **kwargs):
-    deps = load_cur_deps()
+def add(pkg, *args, target=None, **kwargs):
+    deps = load_cur_deps(target=target)
     _add(deps, pkg, *args, **kwargs)
-    write_cur_deps(deps)
+    write_cur_deps(deps, target=target)
     STATE['resolved'] = False
 
 def _add(deps, pkg, uuid=None, **kwargs):
@@ -372,10 +386,10 @@ def _add(deps, pkg, uuid=None, **kwargs):
         for p in pkg:
             _add(deps, p)
 
-def rm(pkg):
-    deps = load_cur_deps()
+def rm(pkg, target=None):
+    deps = load_cur_deps(target=target)
     _rm(deps, pkg)
-    write_cur_deps(deps)
+    write_cur_deps(deps, target=target)
     STATE['resolved'] = False
 
 def _rm(deps, pkg):
@@ -390,3 +404,10 @@ def _rm(deps, pkg):
     else:
         for p in pkg:
             _rm(deps, p)
+
+def offline(value=True):
+    if value is not None:
+        STATE['offline'] = value
+    if value:
+        STATE['resolved'] = False
+    return STATE['offline']
