@@ -28,12 +28,22 @@ def find_julia(compat=None, prefix=None, install=False, upgrade=False):
     As a special case, upgrade=True does not apply when Julia is found in the PATH, because
     if it is already installed then the user is already managing their own Julia versions.
     """
-    # if upgrading, fix compat to the best version
-    orig_compat = compat
+    bestcompat = None
+    if STATE['offline']:
+        upgrade = False
+        install = False
     if upgrade:
-        verstr, _ = best_julia_version(compat)
-        compat = Compat.parse('=='+verstr)
         install = True
+    # env var PYTHON_JULIAPKG_EXE
+    ev_exe = os.getenv('PYTHON_JULIAPKG_EXE')
+    if ev_exe:
+        ev_ver = julia_version(ev_exe)
+        if ev_ver is None:
+            raise Exception(f'PYTHON_JULIAPKG_EXE={ev_exe} is not a Julia executable.')
+        else:
+            if compat is not None and ev_ver not in compat:
+                log(f'WARNING: PYTHON_JULIAPKG_EXE={ev_exe} is Julia {ev_ver} but {compat} is required.')
+            return (ev_exe, ev_ver)
     # first look in the prefix
     if prefix is not None:
         ext = '.exe' if os.name == 'nt' else ''
@@ -41,27 +51,33 @@ def find_julia(compat=None, prefix=None, install=False, upgrade=False):
         pr_ver = julia_version(pr_exe)
         if pr_ver is not None:
             if compat is None or pr_ver in compat:
-                return (pr_exe, pr_ver)
+                if upgrade and bestcompat is None:
+                    bestcompat = Compat.parse('==' + best_julia_version(compat)[0])
+                if bestcompat is None or pr_ver in bestcompat:
+                    return (pr_exe, pr_ver)
     # see if juliaup is installed
     ju_exe = shutil.which('juliaup')
     if ju_exe:
-        ans = ju_find_julia(compat, install=install)
+        if upgrade and bestcompat is None:
+            bestcompat = Compat.parse('==' + best_julia_version(compat)[0])
+        ans = ju_find_julia(bestcompat if upgrade else compat, install=install)
         if ans:
             return ans
     else:
         # see if julia is installed
         jl_exe = shutil.which('julia')
         jl_ver = julia_version(jl_exe)
-        if jl_ver is not None:
-            if orig_compat is None or jl_ver in orig_compat:
-                return (jl_exe, jl_ver)
-            else:
-                log('WARNING: You have Julia installed but it is not compatible with your juliapkg dependencies.')
-                log('  It is recommended that you upgrade Julia or install JuliaUp.')
+        if jl_ver is not None and (compat is None or jl_ver in compat):
+            return (jl_exe, jl_ver)
+        else:
+            log(f'WARNING: You have Julia {jl_ver} installed but {compat} is required.')
+            log(f'  It is recommended that you upgrade Julia or install JuliaUp.')
     # install into the prefix
     if install and prefix is not None:
-        ver, info = best_julia_version(compat)
-        log(f'WARNING: About to install Julia to {prefix}.')
+        if upgrade and bestcompat is None:
+            bestcompat = Compat.parse('==' + best_julia_version(compat)[0])
+        ver, info = best_julia_version(bestcompat if upgrade else compat)
+        log(f'WARNING: About to install Julia {ver} to {prefix}.')
         log(f'  If you use juliapkg in more than one environment, you are likely to have Julia')
         log(f'  installed in multiple locations. It is recommended to install JuliaUp')
         log(f'  (https://github.com/JuliaLang/juliaup) or Julia (https://julialang.org/downloads)')
@@ -69,12 +85,12 @@ def find_julia(compat=None, prefix=None, install=False, upgrade=False):
         install_julia(info, prefix)
         pr_exe = shutil.which(os.path.join(prefix, 'bin', 'julia' + ext))
         pr_ver = julia_version(pr_exe)
-        if pr_ver is not None:
-            if compat is None or pr_ver in compat:
-                return (pr_exe, pr_ver)
-        assert False
+        assert pr_ver is not None
+        assert compat is None or pr_ver in compat
+        assert bestcompat is None or pr_ver in bestcompat
+        return (pr_exe, pr_ver)
     # failed
-    compatstr = '' if orig_compat is None else f' {orig_compat}'
+    compatstr = '' if compat is None else f' {compat}'
     raise Exception(f'could not find Julia{compatstr}')
 
 def ju_find_julia(compat=None, install=False):
